<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

/*
 * Load Additional Data when getting Order by OrderRepository 
 */
class OrderGet
{
    const XML_PATH_ALLOWED_FOR_ORDER = 'sales/gift_options/wrapping_allow_order';
    
    /** @var \FiloBlu\OrderAdditionalData\Model\OrderDataFactory */
    protected $orderDataFactory;

    /** @var \FiloBlu\OrderAdditionalData\Model\ItemDataFactory */
    protected $itemDataFactory;

    /** @var \Magento\Sales\Api\Data\OrderExtensionFactory */
    protected $orderExtensionFactory;

    /** @var \Magento\Sales\Api\Data\OrderItemExtensionFactory */
    protected $orderItemExtensionFactory;
    
     /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;
    
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

     /** 
      * @var \Magento\Framework\Serialize\SerializerInterface 
      */
    protected $_serializer;
    
     /** 
      * @var \FiloBlu\OrderAdditionalData\Model\AdditionalDataFactory
      */    
    protected $_additionalDataFactory;

     /** 
      * @var \FiloBlu\OrderAdditionalData\Model\AdditionalDataFactory
      */    
    protected $_productOptionFactory;

    /**
     * OrderGet constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \FiloBlu\OrderAdditionalData\Model\OrderDataFactory $orderDataFactory
     * @param \FiloBlu\OrderAdditionalData\Model\ItemDataFactory $itemDataFactory
     * @param \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory
     * @param \Magento\Sales\Api\Data\OrderItemExtensionFactory $orderItemExtensionFactory
     * @param \Magento\GiftWrapping\Model\WrappingFactory $wrappingFactory
     * @param \Magento\Framework\Serialize\SerializerInterface $serializer
     * @param \FiloBlu\OrderAdditionalData\Model\AdditionalDataFactory $additionalDataFactory
     * @param \FiloBlu\OrderAdditionalData\Model\ProductOptionFactory $productOptionFactory
     * @param \FiloBlu\OrderAdditionalData\Model\LengowOrderFactory $lengowOrderFactory
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,    
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \FiloBlu\OrderAdditionalData\Model\OrderDataFactory $orderDataFactory,
        \FiloBlu\OrderAdditionalData\Model\ItemDataFactory $itemDataFactory,
        \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory,
        \Magento\Sales\Api\Data\OrderItemExtensionFactory $orderItemExtensionFactory,
        \Magento\GiftWrapping\Model\WrappingFactory $wrappingFactory,
        \Magento\Framework\Serialize\SerializerInterface $serializer,
        \FiloBlu\OrderAdditionalData\Model\AdditionalDataFactory $additionalDataFactory,
        \FiloBlu\OrderAdditionalData\Model\ProductOptionFactory $productOptionFactory,
        \FiloBlu\OrderAdditionalData\Model\LengowOrderFactory $lengowOrderFactory
    ) {
        $this->orderDataFactory = $orderDataFactory;
        $this->itemDataFactory = $itemDataFactory;
        $this->orderExtensionFactory = $orderExtensionFactory;
        $this->orderItemExtensionFactory = $orderItemExtensionFactory;
        $this->wrappingFactory = $wrappingFactory;
        $this->scopeConfig = $context->getScopeConfig();
        $this->_storeManager = $storeManager;
        $this->_serializer = $serializer;
        $this->_additionalDataFactory = $additionalDataFactory;
        $this->_productOptionFactory=$productOptionFactory;
        $this->_lengowOrderFactory=$lengowOrderFactory;
    }

    /**
     * Add Data to order
     *
     * @param \Magento\Sales\Api\OrderRepositoryInterface $subject
     * @param \Magento\Sales\Api\Data\OrderInterface $resultOrder
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    public function afterGet(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        \Magento\Sales\Api\Data\OrderInterface $resultOrder
    ) {
        $resultOrder = $this->getOrderAdditionalData($resultOrder);
        $resultOrder = $this->getOrderItemAdditionalData($resultOrder);
        $resultOrder = $this->getMspData($resultOrder);
        $resultOrder = $this->getLengowData($resultOrder);
        return $resultOrder;
    }

    /**
     * (temporary) Get MSP additional data for order
     *
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    protected function getMspData(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $extensionAttributes = $order->getExtensionAttributes();
        if ($extensionAttributes && $extensionAttributes->getPaymentFeeAmount()) {
            return $order;
        }
        if($order->getPayment()->getMethod()!="msp_cashondelivery"){
            return $order;
        }
        /** @var \Magento\Sales\Api\Data\OrderExtension $orderExtension */
        $orderExtension = $extensionAttributes ? $extensionAttributes : $this->orderExtensionFactory->create();
        $orderExtension->setPaymentFeeAmount(floatval($order->getMspCodAmount())-floatval($order->getMspCodTaxAmount()));
        $orderExtension->setBasePaymentFeeAmount(floatval($order->getBaseMspCodAmount())-floatval($order->getBaseMspCodTaxAmount()));
        $orderExtension->setPaymentTaxAmount(floatval($order->getMspCodTaxAmount()));
        $orderExtension->setBasePaymentTaxAmount(floatval($order->getBaseMspCodTaxAmount()));
        $order->setExtensionAttributes($orderExtension);

        return $order;
    }

    /**
     * (temporary) Get Lengow Data if available
     *
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    protected function getLengowData(\Magento\Sales\Api\Data\OrderInterface $order)
    {        
        if($order->getPayment()->getMethod()!="lengow"){
            return $order;
        }
        /* If lengow module is enabled the function returns the Lengow Order ID. 
         * Otherwise the function is considered a magic function within a Magento\Framework\DataObject and returns null. 
         * */
        $lengowOrderId = $this->_lengowOrderFactory->create()->getLengowOrderIdByOrderId($order->getId());
        if ($lengowOrderId) {
            $lengowOrder = $this->_lengowOrderFactory->create()->load($lengowOrderId);
            
            $extensionAttributes = $order->getExtensionAttributes();
            $orderExtension = $extensionAttributes ? $extensionAttributes : $this->orderExtensionFactory->create();            

            /** @var \Magento\Sales\Api\Data\OrderExtension $orderExtension */
            $orderExtension->setData("lengow_marketplace_name",$lengowOrder->getData('marketplace_name'));
            $orderExtension->setData("lengow_marketplace_sku",$lengowOrder->getData('marketplace_sku'));
            $extra=json_decode($lengowOrder->getData('extra'), true);
            if(is_array($extra) && array_key_exists("processing_fee", $extra)){
                $orderExtension->setData("lengow_marketplace_processing_fee",$extra["processing_fee"]);
            }
            $order->setExtensionAttributes($orderExtension);
        }        
        return $order;
    }

    /**
     * Get additional data for order
     *
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    protected function getOrderAdditionalData(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $extensionAttributes = $order->getExtensionAttributes();
        if ($extensionAttributes && $extensionAttributes->getFbOad()) {
            return $order;
        }
        //$additionalData=$this->objectFactory->create();
        $additionalData=array();
        try {
            /** @var \Magento\AdditionalData\Api\Data\MessageInterface $AdditionalData */
            $orderDataObject = $this->orderDataFactory->create()->load($order->getEntityId(), "order_id");

            if ($orderDataObject && $orderDataObject->getID() > 0 && $orderDataObject->getAdditionalData()!=null ){
                $additionalData=$this->_serializer->unserialize($orderDataObject->getAdditionalData());
            }
        } catch (NoSuchEntityException $e) {
            return $order;
        }

        /** @var \Magento\Sales\Api\Data\OrderExtension $orderExtension */
        $orderExtension = $extensionAttributes ? $extensionAttributes : $this->orderExtensionFactory->create();        
        $additionalDataObject = $this->_additionalDataFactory->create();
        foreach($additionalData as $field => $value){    
            $additionalDataObject->setCustomAttribute($field, $value);
        }
        $order_fields_config=$this->scopeConfig->getValue('oad/general/order_data_fields', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $order_fields=$this->_serializer->unserialize($order_fields_config); 
        if($order_fields){
            foreach($order_fields as $order_field){
                if ($order_field["object_type"]=="order"){                    
                    $additionalDataObject->setCustomAttribute($order_field["fieldname"], $order->getData($order_field["object_field"]));              
                }
            }
        }
        $orderExtension->setFbOad($additionalDataObject);
        $orderExtension->setData("fbnetsuite_base_theoric_shipping_amount", $order->getData("fbnetsuite_base_theoric_shipping_amount"));
        $orderExtension->setData("fbnetsuite_theoric_shipping_amount", $order->getData("fbnetsuite_theoric_shipping_amount"));
        $order->setExtensionAttributes($orderExtension);
        
        try {
            
            $isGiftWrappingAvailableForOrder = $this->scopeConfig->getValue( self::XML_PATH_ALLOWED_FOR_ORDER, \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $this->_storeManager->getStore() );
            if (!$isGiftWrappingAvailableForOrder) {
                throw new \Exception(
                    __('There is no order with provided id or gift wrapping isn\'t allowed')
                );
            }

            $wrappingId = $order->getGwId();
            if (!$wrappingId) {
                throw new \Exception(__('There is no item with provided id in the order'));
            }

            $giftWrapping = $this->wrappingFactory->create()->load($wrappingId);
        
        
        } catch (\Exception $e) {
            return $order;
        }

        /** @var \Magento\Sales\Api\Data\OrderExtension $orderExtension */
        $orderExtension = $extensionAttributes ? $extensionAttributes : $this->orderExtensionFactory->create();
        $orderExtension->setGwId($giftWrapping->getId());
        $orderExtension->setGwAllowGiftReceipt($order->getGwAllowGiftReceipt());
        $orderExtension->setGwAddCard($order->getGwAddCard());
        $orderExtension->setGwPrice($order->getGwPrice());
        $orderExtension->setGwBasePrice($order->getGwBasePrice());
        $orderExtension->setGwItemsPrice($order->getGwItemsPrice());
        $orderExtension->setGwItemsBasePrice($order->getGwItemsBasePrice());
        $orderExtension->setGwCardPrice($order->getGwCardPrice());
        $orderExtension->setGwCardBasePrice($order->getGwCardBasePrice());
        $orderExtension->setGwBaseTaxAmount($order->getGwBaseTaxAmount());
        $orderExtension->setGwTaxAmount($order->getGwTaxAmount());
        $orderExtension->setGwItemsBaseTaxAmount($order->getGwItemsBaseTaxAmount());
        $orderExtension->setGwItemsTaxAmount($order->getGwItemsTaxAmount());
        $orderExtension->setGwCardBaseTaxAmount($order->getGwCardBaseTaxAmount());
        $orderExtension->setGwCardTaxAmount($order->getGwCardTaxAmount());
        $orderExtension->setGwPriceInclTax($order->getGwPriceInclTax());
        $orderExtension->setGwBasePriceInclTax($order->getGwBasePriceInclTax());
        $orderExtension->setGwCardPriceInclTax($order->getGwCardPriceInclTax());
        $orderExtension->setGwCardBasePriceInclTax($order->getGwCardBasePriceInclTax());
        $orderExtension->setGwItemsPriceInclTax($order->getGwItemsPriceInclTax());
        $orderExtension->setGwItemsBasePriceInclTax($order->getGwItemsBasePriceInclTax());
        $order->setExtensionAttributes($orderExtension);

        return $order;
    }

    /**
     * Get additional data for order items
     *
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    protected function getOrderItemAdditionalData(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $orderItems = $order->getItems();
        if (null !== $orderItems) {
            $item_fields=$this->scopeConfig->getValue('oad/general/item_data_fields', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            if($item_fields){
                $item_fields=$this->_serializer->unserialize($item_fields);
                foreach($item_fields as $key=>$item_field){
                    if($item_field["object_type"]!="order_item"){
                        unset($item_fields[$key]);
                    }
                }
            }
            
            /** @var \Magento\Sales\Api\Data\OrderItemInterface $orderItem */
            foreach ($orderItems as $orderItem) {
                $extensionAttributes = $orderItem->getExtensionAttributes();
                if ($extensionAttributes && $extensionAttributes->getFbOad()) {
                    continue;
                }

                try {
                    /* @var \Magento\GiftMessage\Api\Data\MessageInterface $giftMessage */
                    $itemDataObject = $this->itemDataFactory->create()->load($orderItem->getItemId(), "item_id");
                    if(!is_null($itemDataObject->getAdditionalData())){
                        $additionalData=$this->_serializer->unserialize($itemDataObject->getAdditionalData());
                    }else{
                        return $order;
                    }

                } catch (NoSuchEntityException $e) {
                    continue;
                }
                if($item_fields){
                    foreach($item_fields as $item_field){
                        $additionalData[$item_field["fieldname"]] = $orderItem->getData($item_field["object_field"]);
                    }
                }
                /** @var \Magento\Sales\Api\Data\OrderItemExtension $orderItemExtension */
                $orderItemExtension = $extensionAttributes
                    ? $extensionAttributes
                    : $this->orderItemExtensionFactory->create();
                $additionalDataObject = $this->_additionalDataFactory->create();
                foreach($additionalData as $field => $value){   
                    $additionalDataObject->setCustomAttribute($field, $value);
                }
                $orderItemExtension->setFbOad($additionalDataObject);
                $orderItemExtension->setData("kpi_full_price", floatval($orderItem->getData("kpi_full_price")));
                $allProductOptions=$orderItem->getProductOptions();
                if (isset($allProductOptions['options']) && !empty($allProductOptions['options'])) {        
                    $productOptions=$allProductOptions["options"];   
                    $productOptionsArray=array();
                    foreach($productOptions as $key=>$options){
                        $productOptionsArray[$key] = $this->_productOptionFactory->create();
                        foreach($options as $field=>$value){
                            $productOptionsArray[$key]->setData($field,$value);
                        }
                    }   
                    $orderItemExtension->setProductOptions($productOptionsArray);                    
                }
                if(array_key_exists("giftcard_created_codes",$allProductOptions)){
                    $orderItemExtension->setData("giftcard_created_codes",$allProductOptions["giftcard_created_codes"]);
                }
                $orderItem->setExtensionAttributes($orderItemExtension);
            }
        }
        return $order;
    }

    /**
     * @param \Magento\Sales\Api\OrderRepositoryInterface $subject
     * @param \Magento\Sales\Model\ResourceModel\Order\Collection $resultOrder
     * @return \Magento\Sales\Model\ResourceModel\Order\Collection
     */
    public function afterGetList(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        \Magento\Sales\Model\ResourceModel\Order\Collection $resultOrder
    ) {
        /** @var  $order */
        foreach ($resultOrder->getItems() as $order) {
            $this->afterGet($subject, $order);
        }
        return $resultOrder;
    }
}
