<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

use Magento\Framework\Exception\CouldNotSaveException;
/*
 * Save Additional Data when saving Orders by OrderRepository 
 */
class OrderSave
{
    /** @var \FiloBlu\OrderAdditionalData\Model\OrderDataFactory */
    protected $orderDataFactory;

    /** 
      * @var \Magento\Framework\Serialize\SerializerInterface 
      */
    protected $_serializer;

    /**
     * OrderSave constructor.
     * @param \FiloBlu\OrderAdditionalData\Model\OrderDataFactory $orderDataFactory
     * @param \Magento\Framework\Serialize\SerializerInterface $serializer
     */
    public function __construct(
        \FiloBlu\OrderAdditionalData\Model\OrderDataFactory $orderDataFactory,
        \Magento\Framework\Serialize\SerializerInterface $serializer
    ) {
        $this->orderDataFactory = $orderDataFactory;
        $this->_serializer = $serializer;
    }
    
    /**
     * Save additional data on OAD data structures
     *
     * @param \Magento\Sales\Api\OrderRepositoryInterface $subject
     * @param \Magento\Sales\Api\Data\OrderInterface $resultOrder
     * @return \Magento\Sales\Api\Data\OrderInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @throws CouldNotSaveException
     */
    public function afterSave(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        \Magento\Sales\Api\Data\OrderInterface $resultOrder
    ) {
        /** @var \Magento\Sales\Api\Data\OrderInterface $resultOrder */
        $resultOrder = $this->saveOrderAdditionalData($resultOrder);

        return $resultOrder;
    }

    /**
     * Save additional data for order
     *
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return \Magento\Sales\Api\Data\OrderInterface
     * @throws CouldNotSaveException
     */
    protected function saveOrderAdditionalData(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $extensionAttributes = $order->getExtensionAttributes();
        if (null !== $extensionAttributes &&null !== $extensionAttributes->getFbOad()) {
            
            $additionalDataObject = $extensionAttributes->getFbOad();
            $additionalData=array();
            foreach($additionalDataObject->getCustomAttributes() as $customAttribute){
                $additionalData[$customAttribute->getAttributeCode()]=$customAttribute->getValue();
            }
            try {
                $orderData=$this->orderDataFactory->create()->load($order->getEntityId(), "order_id");
                $orderData->setOrderId($order->getEntityId());
                $orderData->setAdditionalData($this->_serializer->serialize($additionalData));
                $orderData->save();                
            } catch (\Exception $e) {
                throw new CouldNotSaveException(
                    __('Could not save Additional Data for order: "%1"', $e->getMessage()),
                    $e
                );
            }
        }
        return $order;
    }

}
